# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: licensing.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0flicensing.proto\"J\n\x0fGetEnclaveReply\x12\x0b\n\x03jwt\x18\x01 \x01(\t\x12\x13\n\x0b\x65nclave_url\x18\x02 \x01(\t\x12\x15\n\rprivate_cloud\x18\x03 \x01(\x08\"$\n\x11GetEnclaveRequest\x12\x0f\n\x07\x61pi_key\x18\x01 \x01(\t2H\n\x10LicensingService\x12\x34\n\nGetEnclave\x12\x12.GetEnclaveRequest\x1a\x10.GetEnclaveReply\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'licensing_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_GETENCLAVEREPLY']._serialized_start=19
  _globals['_GETENCLAVEREPLY']._serialized_end=93
  _globals['_GETENCLAVEREQUEST']._serialized_start=95
  _globals['_GETENCLAVEREQUEST']._serialized_end=131
  _globals['_LICENSINGSERVICE']._serialized_start=133
  _globals['_LICENSINGSERVICE']._serialized_end=205
# @@protoc_insertion_point(module_scope)



class APIKeyException(Exception):
    """
    A custom exception class for wrapping API Key verification errors
    """

    def __init__(self, key, msg):
        self.key = key
        self.msg = msg

    def __str__(self):
        return f"{self.msg}"
    
class PredictionException(Exception):
    """
    A custom exception class for wrapping predictions errors
    """

    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return f"{self.msg}"


from pydantic import BaseModel, validator, ValidationError
from typing import Optional
from .errors import *

from grpc import ssl_channel_credentials

import blind_llama.pb.licensing_pb2 as licensing_pb2
import blind_llama.pb.licensing_pb2_grpc as licensing_pb2_grpc

import os
import grpc
import requests
import json

MITHRIL_SERVICES_URL = os.getenv("MITHRIL_SERVICES_URL", "api.cloud.mithrilsecurity.io")

class BlindLlamaDebugModeWarning(Warning):
    pass

class PromptRequest(BaseModel):
    inputs: str

    @validator("inputs")
    def valid_input(cls, v):
        if not v:
            raise ValidationError("The prompt cannot be empty")
        return v

class Client():
    """A class to represent a connection to a BlindLlama server."""

    def __init__(
        self,
        api_key: str = "",
    ):
        """Connect to the BlindLlama service.
        You needs to provide an API key in order to use the service. 
        Args:
            api_key (str): The API key to use the service. You can either specify an API key here or use the variable environment BLIND_LLAMA_API_KEY. If you specified an API key using the variable environment BLIND_LLAMA_API_KEY, the argument api_key will be ignored. You can get one API key on https://cloud.mithrilsecurity.io/. Default to None.
        Returns:
        """

        channel = grpc.secure_channel(
                MITHRIL_SERVICES_URL,
                ssl_channel_credentials(),
            )

        api_key_env = os.getenv("BLIND_LLAMA_API_KEY", "")
        stub = licensing_pb2_grpc.LicensingServiceStub(channel)
        try:
            enclave_request = licensing_pb2.GetEnclaveRequest(api_key=api_key_env if len(api_key_env) > 0 else api_key)
            response = stub.GetEnclave(enclave_request)
        except grpc.RpcError as rpc_error:
            raise APIKeyException(api_key, rpc_error.details())

        host_ports = response.enclave_url.split(":")
        ports = host_ports[1].split("/")


        if len(response.jwt) > 0: 
            self.jwt = response.jwt
        else:
            raise APIKeyException(
                api_key, "The API Key you provided is invalid."
            )

        self.base_url = f"http://{host_ports[0]}:{ports[1]}"

        super(Client, self).__init__()


    def predict(
        self,
        prompt: str
    ) -> str:
        """Start a prediction on the BlindLlama service.
        Args:
            prompt (str): The prompt on which you want to run a prediction on.
        Returns:
            str: The result of the prediction made by the BlindLlama server
        """

        req = PromptRequest(inputs=prompt)
        Headers = { "accesstoken" : self.jwt }
        resp = requests.post(
            self.base_url + "/",
            json=req.dict(),
            headers=Headers
        )
        ret_json = resp.json()
        generated_text = ""
        for elem in ret_json:
            if "error" in elem:
                raise PredictionException(elem['error'])
            if "generated_text" in elem:
                generated_text += f"{elem['generated_text'].strip()}\n"
        return generated_text.strip()

def connect(api_key: str = "") -> Client:
    """Connect to the BlindLlama service.
    Args:
            api_key (str): The API key to use the service. You can either specify an API key here or use the variable environment BLIND_LLAMA_API_KEY. If you specified an API key using the variable environment BLIND_LLAMA_API_KEY, the argument api_key will be ignored. You can get one API key on https://cloud.mithrilsecurity.io/. Default to None.
   
    Returns:
        Client: A connection to the BlindLlama service.
    """
    return Client(api_key)
